--[[
	Bagnon Database
		Functions for storing and accessing player inventory data
		This is intended to be completely seperate from Bagnon_Forever
		
	BagnonDB has this format, which was adapted from KC_Items
		Realm
			Character
				Bag
					size,count,[link]
					ItemSlot
						link,[count]
				Money
--]]

--local globals
local currentPlayer = UnitName("player"); --the current player that's logged on
local currentRealm = GetRealmName(); --what currentRealm we're in

function BagnonDB_OnLoad()
	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("PLAYER_LOGIN");
	this:RegisterEvent("BANKFRAME_CLOSED");
	this:RegisterEvent("BANKFRAME_OPENED");
	this:RegisterEvent("PLAYERBANKSLOTS_CHANGED");
	this:RegisterEvent("PLAYER_MONEY");
end

function BagnonDB_OnEvent(event)
	if(event == "BAG_UPDATE") then
		BagnonDB_SaveBagData(arg1);
	elseif(event == "PLAYERBANKSLOTS_CHANGED") then
		BagnonDB_SaveBagData(-1);
	elseif(event == "BANKFRAME_CLOSED") then
		atBank = nil;
	elseif(event == "BANKFRAME_OPENED") then
		atBank = 1;
		BagnonDB_SaveBankData();
	elseif(event == "PLAYER_MONEY" or event == "PLAYER_LOGIN") then
		BagnonDB_SavePlayerMoney();
	elseif(event == "ADDON_LOADED" and arg1 == "Bagnon_Forever") then
		this:UnregisterEvent("ADDON_LOADED");
		BagnonDB_LoadVariables();
	end
end

function BagnonDB_LoadVariables()
	local currentRealm = GetRealmName(); --what currentRealm we're in
	
	if(not BagnonDB) then
		BagnonDB = {};
		BagnonDB.version = BAGNON_FOREVER_VERSION;
	end
	
	if(not BagnonDB[currentRealm]) then
		BagnonDB[currentRealm] = {};
	end
	
	if(not BagnonDB[currentRealm][UnitName("player")]) then
		BagnonDB[currentRealm][UnitName("player")] = {};
	end
	
	if(BagnonDB.version ~= BAGNON_FOREVER_VERSION) then
		BagnonDB_UpdateVersion();
	end
end

function BagnonDB_UpdateVersion()
	BagnonDB.version = BAGNON_FOREVER_VERSION;
	
	DEFAULT_CHAT_FRAME:AddMessage(BAGNON_FOREVER_UPDATED, 1, 1, 0);
end

--[[ 
	Storage Functions
--]]

function BagnonDB_SaveAllData()
	local i;
	--you know, this should probably be a constant
	for i = -2, 10, 1 do
		BagnonDB_SaveBagData(i);
	end
	BagnonDB_SavePlayerMoney();
end

function BagnonDB_SaveBankData()
	BagnonDB_SaveBagData(-1);
	local bagID;
	for bagID = 5, 10, 1 do
		BagnonDB_SaveBagData(bagID);
	end
end

--called whenever the bag updates, save all the bag's dat and
function BagnonDB_SaveBagData(bagID)
	--don't save bank data unless you're at the bank
	if( Bagnon_IsBankBag(bagID) and not atBank ) then
		return;
	end
	
	local size;
	if(bagID == KEYRING_CONTAINER) then
		size = GetKeyRingSize();
	else
		size = GetContainerNumSlots(bagID);
	end
	
	if(size > 0) then
		local link, count;
		
		if(bagID > 0) then
			link = BagnonDB_ToShortLink( GetInventoryItemLink("player", ContainerIDToInventoryID(bagID) ) );
		end
		
		count = GetInventoryItemCount("player", ContainerIDToInventoryID(bagID));
		
		--save bag size
		BagnonDB[currentRealm][currentPlayer][bagID] = {};
		BagnonDB[currentRealm][currentPlayer][bagID].s = size .. "," .. count .. ",";
		
		if(link) then
			BagnonDB[currentRealm][currentPlayer][bagID].s = BagnonDB[currentRealm][currentPlayer][bagID].s .. link;
		end

		--save all item info
		for index = 1, size, 1 do
			BagnonDB_SaveItemData(bagID, index);
		end
	else
		BagnonDB[currentRealm][currentPlayer][bagID] = nil;
	end
end

function BagnonDB_SaveItemData(bagID, itemSlot)
	local texture, count = GetContainerItemInfo(bagID, itemSlot);
	local data;
	
	if(texture) then
		data = BagnonDB_ToShortLink( GetContainerItemLink(bagID, itemSlot) ) .. ",";
		if(count > 1) then
			data = data .. count;
		end
	end
	
	BagnonDB[currentRealm][currentPlayer][bagID][itemSlot] = data;
end

function BagnonDB_SavePlayerMoney()
	BagnonDB[currentRealm][currentPlayer].g = GetMoney();
end

--[[
	Access  Functions
--]]

--returns <player>'s current money
function BagnonDB_GetMoney(player)
	return BagnonDB[currentRealm][player].g;
end

function BagnonDB_GetBagData(player, bagID)
	local data = BagnonDB[currentRealm][player][bagID];
	if(data) then
		local _, _, size, count, link = string.find(data.s, "([%w_:]+),([%w_:]+),([%w_:]*)");
		
		if(link and link ~="") then
			return size, "item:" .. link, tonumber(count);
		elseif(size) then
			return size, nil, tonumber(count);
		end
	end
	return 0;
end

--returns the item link, (in the form of item:www:xxx:yyy:zzz),  texture (full path) , and count of <player>'s <item>
function BagnonDB_GetItemDataFromSlot(player, bagID, slot)
	if( not( BagnonDB[currentRealm] and BagnonDB[currentRealm][player] and BagnonDB[currentRealm][player][bagID] )) then
		return nil;
	end
	local itemData = BagnonDB[currentRealm][player][bagID][slot];
	
	if(itemData) then
		local _, _, shortLink, count = string.find(itemData, "([%w_:]+),([%w_:]*)");
		
		shortLink = "item:" .. shortLink;
		
		if(count == "") then
			count = 1;
		else
			count = tonumber(count);
		end
		
		local _, _, _, _, _, _, _, _, texture = GetItemInfo(shortLink);
		
		return shortLink, texture, count;
	end
	
	return nil;
end

--returns the item link, (in the form of item:www:xxx:yyy:zzz),  texture (full path) , and count of <player>'s <item>
function BagnonDB_GetItemData(item)
	local itemData = BagnonDB[currentRealm][item:GetParent():GetParent().player][item:GetParent():GetID()][item:GetID()];
	
	if(itemData) then
		local _, _, shortLink, count = string.find(itemData, "([%w_:]+),([%w_:]*)");
		
		if(shortLink ~= "") then
			shortLink = "item:" .. shortLink;
		else
			shortLink = nil;
		end
		
		if(count == "") then
			count = 1;
		else
			count = tonumber(count);
		end
		
		local _, _, _, _, _, _, _, _, texture = GetItemInfo(shortLink);
		
		return shortLink, texture, count;
	end
	
	return nil;
end

--return the full hyperlink of an item
function BagnonDB_GetFullItemLink(item)
	local link = ( BagnonDB_GetItemData(item) );
	
	if(link) then
		local name, ilink, quality = GetItemInfo( link );
		local r,g,b,hex = GetItemQualityColor( quality );
	
		return hex .. "|H".. link .. "|h[" .. name .. "]|h|r";
	end
end

--[[
	Removal Functions
--]]

function BagnonDB_RemovePlayer(player, realm)
	if(BagnonDB[realm]) then
		BagnonDB[realm][player] = nil;
	end
end

--[[
	Conversion Functions 
--]]

--takes a full item hyperlink and returns the www:xxx:yyy:zzz form
function BagnonDB_ToShortLink(fullLink)
	if(fullLink) then
		local _, _, w, x, y, z = string.find(fullLink, "(%d+):(%d+):(%d+):(%d+)") ;
		return w .. ":" .. x .. ":" .. y .. ":" .. z;
	end
	return nil;
end

function BagnonDB_ToID(link)
	if(link) then
		local _, _, id = string.find(link, "(%d+)");
		return id;
	end
end